const lo = require('lodash');
const hlp = require('../../../helpers');
const hlpPolicy = require('../../../helpers_policy');
const hlpSch = require('./TaskScheduleHelper.js');
const utilsLI = require('../../utils/li.js');

const defPolicySections = ['LogInspector','PredefinedTasksPolicy']
const formCustomRulesValues = ['CheckWinEvtLog','WindowsEventLog']
const formPredefRulesValues = ['UsePredefRules','PredefRules']

function toView(TASK_ADDITIONAL_PARAMS, activePolicyCache, formType) {
    if(formType === 'Schedule' ) {
        return hlpSch.toView(TASK_ADDITIONAL_PARAMS.Schedule, activePolicyCache.PredefinedTasksPolicy.LogInspectorStartOptions);
    }
    if(formType!=='Main')
        return {}

    let data = {
        Form_LogInspect_CustomRules:hlp.localSettingsToView(TASK_ADDITIONAL_PARAMS.Settings,activePolicyCache.LogInspector,formCustomRulesValues),
        Form_LogInspect_PredefinedRules: hlp.localSettingsToView(TASK_ADDITIONAL_PARAMS.Settings,activePolicyCache.LogInspector,formPredefRulesValues)
    }
    data.Form_LogInspect_CustomRules = utilsLI.toViewCustomRules(data.Form_LogInspect_CustomRules);
    data.Form_LogInspect_PredefinedRules = utilsLI.toViewPredefinedRules(data.Form_LogInspect_PredefinedRules);
    data.Form_LogInspect_CustomRules.LocalTask = true;
    data.Form_LogInspect_PredefinedRules.LocalTask = true;
    return data
}

function toRaw(TASK_ADDITIONAL_PARAMS, changedData, formType) {
    if(formType === 'Schedule' ) {
        hlpSch.toRaw(changedData, TASK_ADDITIONAL_PARAMS.Schedule);
        return;
    }
    else if(formType==='Main') {
        // custom rules
        const CustomRules = utilsLI.toRowLocalCustomRules(changedData.Form_LogInspect_CustomRules);
        Object.assign(TASK_ADDITIONAL_PARAMS.Settings, lo.pick(CustomRules, formCustomRulesValues));
        // predefined rules
        const PredRules = utilsLI.toRowLocalPredefinedRules(changedData.Form_LogInspect_PredefinedRules);
        Object.assign(TASK_ADDITIONAL_PARAMS.Settings, lo.pick(PredRules, formPredefRulesValues));
    }
}

function RbacToPolicyCache(activePolicyCache){
    const allFields = formCustomRulesValues.concat(formPredefRulesValues);
    activePolicyCache.LogInspector = hlpPolicy.SetPolicyFieldsRbacLocked(activePolicyCache.LogInspector, allFields);
    activePolicyCache.PredefinedTasksPolicy = hlpPolicy.SetPolicyFieldsRbacLocked(activePolicyCache.PredefinedTasksPolicy, ['LogInspectorStartOptions']);
    return activePolicyCache;
}

module.exports = {
    defPolicySections,
    toView,
    toRaw,
    RbacToPolicyCache
};
