const lo = require('lodash');
const hlp = require('../../../helpers');
const hlpPolicy = require('../../../helpers_policy');
const hlpSch = require('./TaskScheduleHelper.js');
const utils    = require('../../../utils');
const utilsPep = require('../../utils/pep.js');

const defPolicySections = ['ApplicationControlPolicy','PredefinedTasksPolicy']
const formCommonValues = ['EnforsmentMode','UseCache','BlockInterperetesWhenCliEmpty',
    'UseBinaryRules','MonitorDllLoading','UseScriptOrMsiRules',
    'BlockUntrustedApplication', 'AllowTrustedApplication','AllowUserOrGroups']
const formRulesValues = ['RuleUsageStrategy','RuleCollection','AdditionalRuleCollection']
const formSdcValues = ['UseTrustedInstaller', 'TrustWindowsInstaller', 'TrustBackgroundIntelligentTransferService', 'TrustedInstallers']


function toView(TASK_ADDITIONAL_PARAMS, activePolicyCache, formType) {
    if(formType === 'Schedule' ) {
        return hlpSch.toView(TASK_ADDITIONAL_PARAMS.Schedule, activePolicyCache.PredefinedTasksPolicy.ApplicationControlStartOptions);
    }
    if(formType!=='Main')
        return {}

    let data = {
        Form_Control_PepCommon: hlp.localSettingsToView(TASK_ADDITIONAL_PARAMS.Settings,activePolicyCache.ApplicationControlPolicy,formCommonValues),
        Form_Control_PepRules: hlp.localSettingsToView(TASK_ADDITIONAL_PARAMS.Settings,activePolicyCache.ApplicationControlPolicy,formRulesValues),
        Form_Control_PepSdc: hlp.localSettingsToView(TASK_ADDITIONAL_PARAMS.Settings,activePolicyCache.ApplicationControlPolicy,formSdcValues)
    }

    data.Form_Control_PepCommon.Users = utils.toViewArray(data.Form_Control_PepCommon.AllowUserOrGroups.KLPRSS_Val[0], 'User');
    data.Form_Control_PepRules.uiFields = utilsPep.toViewSettingsRules(data.Form_Control_PepRules);

    data.Form_Control_PepCommon.LocalTask = true;
    data.Form_Control_PepRules.LocalTask = true;
    data.Form_Control_PepSdc.LocalTask = true;
    return data
}

function toRaw(TASK_ADDITIONAL_PARAMS, changedData, formType) {
    if(formType === 'Schedule' ) {
        hlpSch.toRaw(changedData, TASK_ADDITIONAL_PARAMS.Schedule);
        return;
    }
    if(formType!=='Main')
        return;
    // common
    const Common = hlp.ItemToLocalSettings(changedData.Form_Control_PepCommon);
    Object.assign(TASK_ADDITIONAL_PARAMS.Settings, lo.pick(Common, formCommonValues));
    // rules
//    changedData.Form_Control_PepRules = utilsPep.toRowRules(changedData.Form_Control_PepRules);
//    const Rules = hlp.ItemToLocalSettings(changedData.Form_Control_PepRules);
//    Object.assign(TASK_ADDITIONAL_PARAMS.Settings, lo.pick(Rules, formRulesValues));

    // sdc
    const Sdc = hlp.ItemToLocalSettings(changedData.Form_Control_PepSdc);
    Object.assign(TASK_ADDITIONAL_PARAMS.Settings, lo.pick(Sdc, formSdcValues));
}

function RbacToPolicyCache(activePolicyCache){
    const allFields = formCommonValues.concat(formRulesValues).concat(formSdcValues);
    activePolicyCache.ApplicationControlPolicy = hlpPolicy.SetPolicyFieldsRbacLocked(activePolicyCache.ApplicationControlPolicy, allFields);
    activePolicyCache.PredefinedTasksPolicy = hlpPolicy.SetPolicyFieldsRbacLocked(activePolicyCache.PredefinedTasksPolicy, ['ApplicationControlStartOptions']);
    return activePolicyCache;
}

module.exports = {
    defPolicySections,
    toView,
    toRaw,
    RbacToPolicyCache
};
