const lo = require('lodash')
const hlp = require('../../helpers')
const hlpPolicy = require('../../helpers_policy')
const consts = require('../../consts')
const settConverters = require('./sections/index.js')
const settCommonData = require('./consts.js')

function Settings () {
    let changesList = [];
    let settingsCache = {};
    let policiesCache = {};
    let rbacCache = {};
    let curHostId = '';
    let groupId = 0;
    let activePolicyId = -1;

    function AddToChangesList (arrSections) {
        arrSections = Array.isArray(arrSections) ? arrSections : [arrSections]
        for (let section of arrSections) {
            if (section!=='' && !changesList.includes(section)) {
                changesList.push(section)
            }
        }
    }
    /*
     * Метод добавляет в cache политик новые секции
     */
    async function UpdateSettingsCache (api, arrSections, hostId) {
        arrSections = Array.isArray(arrSections) ? arrSections : [arrSections]
        for (let section of arrSections) {
            if( consts.settSsSections[section]=== undefined || settingsCache[section]!==undefined)
                continue
            let sectionData = {}
            try{
              sectionData = await api.HostGroup.SS_Read({
                                strHostName: hostId,
                                strType: 'SS_SETTINGS',
                                strProduct: consts.wstrProduct,
                                strVersion: consts.wstrVersion,
                                strSection: consts.settSsSections[section] })
            }
            catch (error) {
                console.error(error)
            }
            let settData = (sectionData.result!== undefined && sectionData.result.PxgError===undefined) ? sectionData.result : settCommonData.Defaults[section];
//            await GetRbacCache (api, section, settData);
            settingsCache[section] = settData;
        }
    }
    async function UpdatePolicyRbacCache (api, arrSections) {
        arrSections = Array.isArray(arrSections) ? arrSections : [arrSections]
        for (let section of arrSections) {
            if( consts.policySsSections[section]=== undefined || policiesCache[section]===undefined)
                continue;
            const policyData = hlp.serialize(policiesCache[section]);
            let ReadonlyNodes = {};
            const ssPolicySection = consts.policySsSections[section];
            try{
                ReadonlyNodes = await api.HstAccessControl.GetSettingsReadonlyNodes({
                    lGroupId: groupId,
                    szwProduct: consts.wstrProduct,
                    szwVersion: consts.wstrVersion,
                    szwSectionName: ssPolicySection,
                    pSettingsSection:policyData
                });
                if (ReadonlyNodes.result !== undefined) {
                    policiesCache[section] = hlpPolicy.SetPolicyFieldsRbacLocked(policiesCache[section], ReadonlyNodes.result[ssPolicySection]);
                }
            }
            catch (error) {
                console.error(error);
            }
        }
    }

    /*
     * Called via api.js in getFormModel
     */
    async function getSettingsData (api, section, hostId, sGroupId) {
        if(curHostId != hostId) {
            curHostId = hostId;
            groupId = parseInt(sGroupId);
            changesList = [];
            settingsCache = {};
            policiesCache = {};
        }
        activePolicyId = await hlpPolicy.GetHostActivePolicyId(api, hostId);

        if (section === undefined || settConverters[section] === undefined)
            return {}

        if (settConverters[section].sections !== undefined) {
            await UpdateSettingsCache(api, settConverters[section].sections, hostId);
            if(activePolicyId===-1){
                hlpPolicy.UpdatePoliciesCacheAsDefault(settConverters[section].sections, policiesCache, settCommonData.PolicyMappper);
            } else {
                await hlpPolicy.UpdatePoliciesCacheEx(api, activePolicyId, settConverters[section].sections, policiesCache, settCommonData.PolicyMappper);
            }
            await UpdatePolicyRbacCache (api, settConverters[section].sections);
        }

        return (settConverters[section].toView !== undefined)  ? settConverters[section].toView(settingsCache,policiesCache) : {}
    }
    /*
     * Called via api.js in setFormModel
     */
    async function updateSettingsData (kscApi, section, data) {
        if (section === undefined || settConverters[section] === undefined)
            return;
        if (settConverters[section].sections !== undefined) {
            AddToChangesList(settConverters[section].sections);
        }
        if(settConverters[section].toRawEx!==undefined)
            await settConverters[section].toRawEx(kscApi, data, settingsCache);
        else
            settConverters[section].toRaw(data, settingsCache);
    }

    /*
     * Called via api.js in beforeApplyChanges
     */
    function commitSettingsData (api) {
    }

    /*
     * Called via api.js in applyChanges
     */
    async function saveSettingsData (api, hostId) {
        if(changesList.length === 0 )
            return;
        if(!hostId)
            hostId = curHostId;

        for (let section of changesList) {
            try{
                await api.HostGroup.SS_Write({
                    strHostName: hostId,
                    strType: 'SS_SETTINGS',
                    strProduct: consts.wstrProduct,
                    strVersion: consts.wstrVersion,
                    strSection: consts.settSsSections[section],
                    nOption: 3,
                    pSettings: settingsCache[section]
                })
            }
            catch (error) {
                console.error(error)
            }
        }

        changesList = [];
        settingsCache = {};
    }

    return {
        getSettingsData,
        updateSettingsData,
        commitSettingsData,
        saveSettingsData
    }
}

module.exports = Settings
