const lo = require('lodash');
const crypto = require('crypto');
const buffer = require('buffer');
const hlp = require('../../../../helpers')

const sections = ['BLSettings', 'PasswordSettings']
const policyBlValues = ['OdsUseSelfRecovery', 'OdsSelfRecoveryAttempts', 'DontStartTasksIfNoSupply', 'StopScansIfNoSupply']
const localBlValues = ['UseSelfRecovery', 'SelfRecoveryAttempts', 'DontStartTasksIfNoSupply', 'StopScansIfNoSupply']
const pswrdValues = ['PasswordHash', 'PasswordSalt', 'UsePassword']

function toView(settCache, activePolicyCache) {
    // fix different values in policy and local settings
    let blSettTemp = settCache.BLSettings;
    blSettTemp.OdsUseSelfRecovery = settCache.BLSettings.UseSelfRecovery;
    blSettTemp.OdsSelfRecoveryAttempts = settCache.BLSettings.SelfRecoveryAttempts;

    let data = hlp.localSettingsToView(blSettTemp,activePolicyCache.BLSettings,policyBlValues)
    data.PasswordSett = {
        KLPRSS_Val: hlp.localSettingsToView(settCache.PasswordSettings,activePolicyCache.PasswordSettings,pswrdValues)
    }
    data.NewPassword=''
    data.PasswordWasSet = data.PasswordSett.KLPRSS_Val.PasswordHash.KLPRSS_Val[0]!=='' ? true : false
    data.LocalSettings = true
    return data
}

function toRaw(changedData, settCache) {
    hlp.ItemToLocalSettings(changedData.PasswordSett.KLPRSS_Val);
    hlp.ItemToLocalSettings(changedData);
    changedData.UseSelfRecovery = changedData.OdsUseSelfRecovery;
    changedData.SelfRecoveryAttempts = changedData.OdsSelfRecoveryAttempts;

    if( changedData.PasswordSett.PasswordSalt==='' ){
        const buf = Buffer.alloc(64);
        changedData.PasswordSett.PasswordSalt = crypto.randomFillSync(buf).toString('hex');
    }
    if(changedData.PasswordSett.UsePassword===true && changedData.NewPassword!==''){
        const saltBuff = Buffer.from(changedData.PasswordSett.PasswordSalt, 'hex');
        const pswBytes = Buffer.from(changedData.NewPassword, 'utf-8');
        const pswBytes16 = buffer.transcode(pswBytes, 'utf8', 'utf16le');
        const pswrd = crypto.pbkdf2Sync(pswBytes16, saltBuff, 1000, 64, 'sha512');
        if(pswrd)
            changedData.PasswordSett.PasswordHash = pswrd.toString('hex');
    }
    else{
        changedData.PasswordSett.PasswordHash = '';
    }

    Object.assign(settCache.PasswordSettings, lo.pick(changedData.PasswordSett, pswrdValues))
    Object.assign(settCache.BLSettings, lo.pick(changedData, localBlValues))
}

module.exports = {
    sections,
    toView,
    toRaw
};
