const utils = require('../../utils');
const hlp_time = require('../../helpers_time');
const hlp_cmn = require('./common');

const LiTimeIntervalEnum = {Allowed:1, Forbidden:2};

function GetSecondsFromParams(data) {
    let actual_value = (data.value === undefined) ? data : ((data.value.KLPRSS_Val === undefined) ? data.value : data.value.KLPRSS_Val);
    return (typeof actual_value === 'string') ? parseInt(actual_value) : '';
}

function toViewCustomRules(data) {
    for(let item of data.WindowsEventLog.KLPRSS_Val[0]){
        item.uiFields = {
            EventIDs:utils.toViewArray(item.EventToInspect.KLPRSS_Val[0].EventIDs.KLPRSS_Val,'EventId')
        }
    }
    return data;
}

function toViewPredefinedRules(data) {
    data.Form_LogInspectHeurAdvanced = {}
    data.Form_LogInspectHeurUsersExcl = {Users:[]}
    data.Form_LogInspectHeurIpsExcl = {Ips:[]}

    for(let rule of data.PredefRules.KLPRSS_Val[0]){
        const options = rule.RuleOptions.KLPRSS_Val
        if (options.UseCountInPeriod.KLPRSS_Val) {
            data.Form_LogInspectHeurAdvanced.NumberOfAttempts = options.CountInPeriod.KLPRSS_Val.Count.KLPRSS_Val;
            data.Form_LogInspectHeurAdvanced.DuringSeconds    = options.CountInPeriod.KLPRSS_Val.Period.KLPRSS_Val;
        }
        if (options.UseTimeInterval.KLPRSS_Val && options.TimeInterval.KLPRSS_Val.IntervalType.KLPRSS_Val=== LiTimeIntervalEnum.Forbidden) {
            data.Form_LogInspectHeurAdvanced.UseTimeControls = true;
            const startSeconds = GetSecondsFromParams(options.TimeInterval.KLPRSS_Val.StartTime.KLPRSS_Val);
            data.Form_LogInspectHeurAdvanced.From = hlp_time.SecondsToTimeStr(startSeconds);
            const stopSeconds = GetSecondsFromParams(options.TimeInterval.KLPRSS_Val.StopTime.KLPRSS_Val);
            data.Form_LogInspectHeurAdvanced.To = hlp_time.SecondsToTimeStr(stopSeconds);
        }
        if (options.UseTrustedUsers.KLPRSS_Val===true) {
            data.Form_LogInspectHeurUsersExcl.Users = utils.toViewArray(options.TrustedUsers.KLPRSS_Val,'User');
        }
        if (options.UseTrustedIPs.KLPRSS_Val===true) {
            data.Form_LogInspectHeurIpsExcl.Ips = utils.toViewArray(options.TrustedIPs.KLPRSS_Val,'Ip');
        }
    }
    return data;
}

function toRowCustomRules(changedData) {
    for(let item of changedData.WindowsEventLog.KLPRSS_Val[0]){
        item.EventToInspect.KLPRSS_Val[0].EventIDs.KLPRSS_Val = utils.toRawArray(item.uiFields.EventIDs,'EventId');
        delete item.uiFields;
    }
    return changedData;
}

function toRowLocalCustomRules(changedData) {
    let welRules = [];
    for(let item of changedData.WindowsEventLog.KLPRSS_Val[0]){
        let rule = {
            EventToInspect: [{
                EventIDs:utils.toRawArray(item.uiFields.EventIDs,'EventId'),
                Name: item.EventToInspect.KLPRSS_Val[0].Name.KLPRSS_Val,
                TreatAsCritical:item.EventToInspect.KLPRSS_Val[0].TreatAsCritical.KLPRSS_Val,
                __VersionInfo : [1, 0]
            }],
            Source: item.Source.KLPRSS_Val,
            __VersionInfo : [1, 0]
        };
        welRules.push(rule);
    }
    return {
        CheckWinEvtLog: changedData.CheckWinEvtLog.KLPRSS_Val[0],
        WindowsEventLog: welRules
    };
}

function toRowPredefinedRules(changedData) {
    for(let rule of changedData.PredefRules.KLPRSS_Val[0]){
        let options = rule.RuleOptions.KLPRSS_Val
        if (options.UseCountInPeriod.KLPRSS_Val) {
            options.CountInPeriod.KLPRSS_Val.Count.KLPRSS_Val = changedData.Form_LogInspectHeurAdvanced.NumberOfAttempts;
            options.CountInPeriod.KLPRSS_Val.Period.KLPRSS_Val = changedData.Form_LogInspectHeurAdvanced.DuringSeconds;
        }
        if (options.UseTimeInterval.KLPRSS_Val && options.TimeInterval.KLPRSS_Val.IntervalType.KLPRSS_Val=== LiTimeIntervalEnum.Forbidden) {
            const startSeconds = hlp_time.TimeStrToSeconds(changedData.Form_LogInspectHeurAdvanced.From);
            options.TimeInterval.KLPRSS_Val.StartTime.KLPRSS_Val.value = startSeconds.toString();
            const stopSeconds = hlp_time.TimeStrToSeconds(changedData.Form_LogInspectHeurAdvanced.To);
            options.TimeInterval.KLPRSS_Val.StopTime.KLPRSS_Val.value = stopSeconds.toString();
        }
        if (options.UseTrustedUsers.KLPRSS_Val===true) {
            options.TrustedUsers.KLPRSS_Val = utils.toRawArray(changedData.Form_LogInspectHeurUsersExcl.Users,'User');
        }
        if (options.UseTrustedIPs.KLPRSS_Val===true) {
            options.TrustedIPs.KLPRSS_Val = utils.toRawArray(changedData.Form_LogInspectHeurIpsExcl.Ips,'Ip');
        }
    }
    return changedData;
}

function toRowLocalPredefinedRules(changedData) {
    let predefRules = [];
    for(let rule of changedData.PredefRules.KLPRSS_Val[0]){
        let local_rule = hlp_cmn.ItemToLocalSettings(rule);
        if (local_rule.RuleOptions.UseCountInPeriod) {
            local_rule.RuleOptions.CountInPeriod.Count = changedData.Form_LogInspectHeurAdvanced.NumberOfAttempts;
            local_rule.RuleOptions.CountInPeriod.Period = changedData.Form_LogInspectHeurAdvanced.DuringSeconds;
        }
        if (local_rule.RuleOptions.UseTimeInterval && local_rule.RuleOptions.TimeInterval.IntervalType=== LiTimeIntervalEnum.Forbidden) {
            const startSeconds = hlp_time.TimeStrToSeconds(changedData.Form_LogInspectHeurAdvanced.From);
            local_rule.RuleOptions.TimeInterval.StartTime.value = startSeconds.toString();
            const stopSeconds = hlp_time.TimeStrToSeconds(changedData.Form_LogInspectHeurAdvanced.To);
            local_rule.RuleOptions.TimeInterval.StopTime.value = stopSeconds.toString();
        }
        if (local_rule.RuleOptions.UseTrustedUsers===true) {
            local_rule.RuleOptions.TrustedUsers = utils.toRawArray(changedData.Form_LogInspectHeurUsersExcl.Users,'User');
        }
        if (local_rule.RuleOptions.UseTrustedIPs===true) {
            local_rule.RuleOptions.TrustedIPs = utils.toRawArray(changedData.Form_LogInspectHeurIpsExcl.Ips,'Ip')
        }
        predefRules.push(local_rule);
    }
    return {
        UsePredefRules:changedData.UsePredefRules.KLPRSS_Val[0],
        PredefRules: predefRules
    };
}

module.exports = {
    toViewCustomRules,
    toViewPredefinedRules,
    toRowCustomRules,
    toRowLocalCustomRules,
    toRowPredefinedRules,
    toRowLocalPredefinedRules
}