const taskDefs = require('../../default/tasks.js')
const utils = require('../../utils')

var OdsActionEnum = {Cure:1, Remove:2, Recommended:3, Report:4, Quarantine: 5,Skip: 7,Postpone:8}
const InfAA = {cure:0, cure_delete:1, delete:2,recommended:3,notify:100 }
const SuspAA = { quarantine: 0, delete:1, recommended:2,notify:100 }


function getUIFields(scanSett) {
    const out = {
        InfAction:InfAA.notify,
        SuspAction: SuspAA.quarantine,
        ExtList: scanSett.ScanExtensionMasks.join(),
        ExcludeMasks: scanSett.ExcludeMasks.join(),
        ExcludeThreats:scanSett.ExcludeThreats.join(),
        Bigger: scanSett.SizeLimit/(1024*1024)
    }

    switch(scanSett.InfectedFirstAction.ActionType)  {
        case OdsActionEnum.Cure:
            out.infAction = scanSett.InfectedSecondAction.ActionType===OdsActionEnum.Remove ? InfAA.cure_delete : InfAA.cure;
            break;
        case OdsActionEnum.Remove:
            out.infAction = InfAA.delete;
            break;
        case OdsActionEnum.Recommended:
            out.infAction = InfAA.recommended;
            break;
    }

    switch(scanSett.SuspiciousFirstAction.ActionType)  {
        case OdsActionEnum.Info:
        case OdsActionEnum.Skip:
        case OdsActionEnum.Report:
            out.suspAction = SuspAA.notify;
            break;
        case OdsActionEnum.Quarantine:
            out.suspAction = SuspAA.quarantine;
            break;
        case OdsActionEnum.Remove:
            out.suspAction = SuspAA.delete;
            break;
        case OdsActionEnum.Recommended:
            out.suspAction = SuspAA.recommended;
            break;
    }
    return out
}

function UIFieldsToSettings(scanSett, uiFields) {
    if(uiFields===undefined)
        scanSett;

    scanSett.ScanExtensionMasks = utils.normalizedArrayMasks(uiFields.ExtList);
    scanSett.ExcludeMasks = utils.normalizedArrayMasks(uiFields.ExcludeMasks);
    scanSett.ExcludeThreats = utils.normalizedArrayMasks(uiFields.ExcludeThreats);
    scanSett.SizeLimit = uiFields.Bigger*(1024*1024)

    switch (uiFields.InfAction) {
        case InfAA.cure:
            scanSett.InfectedFirstAction.ActionType = OdsActionEnum.Cure;
            scanSett.InfectedSecondAction.ActionType= OdsActionEnum.Report;
            break;
        case InfAA.cure_delete:
            scanSett.InfectedFirstAction.ActionType = OdsActionEnum.Cure;
            scanSett.InfectedSecondAction.ActionType= OdsActionEnum.Remove;
            break;
        case InfAA.delete:
            scanSett.InfectedFirstAction.ActionType = OdsActionEnum.Remove;
            scanSett.InfectedSecondAction.ActionType= OdsActionEnum.Report;
            break;
        case InfAA.recommended:
            scanSett.InfectedFirstAction.ActionType = OdsActionEnum.Recommended;
            scanSett.InfectedSecondAction.ActionType= OdsActionEnum.Recommended;
            break;
        case InfAA.notify:
        default:
            scanSett.InfectedFirstAction.ActionType = OdsActionEnum.Report;
            scanSett.InfectedSecondAction.ActionType= OdsActionEnum.Report;
            break;
    }

    switch (uiFields.SuspAction) {
        case SuspAA.quarantine:
            scanSett.SuspiciousFirstAction.ActionType = OdsActionEnum.Quarantine;
            scanSett.SuspiciousSecondAction.ActionType= OdsActionEnum.Report;
            break;
        case SuspAA.delete:
            scanSett.SuspiciousFirstAction.ActionType = OdsActionEnum.Remove;
            scanSett.SuspiciousSecondAction.ActionType= OdsActionEnum.Report;
            break;
        case SuspAA.notify:
            scanSett.SuspiciousFirstAction.ActionType = OdsActionEnum.Report;
            scanSett.SuspiciousSecondAction.ActionType= OdsActionEnum.Report;
            break;
        case SuspAA.recommended:
        default:
            scanSett.SuspiciousFirstAction.ActionType = OdsActionEnum.Recommended;
            scanSett.SuspiciousSecondAction.ActionType= OdsActionEnum.Recommended;
            break;
    }

    return scanSett;
}

function toFormsOnDemand(sett) {
    let scanScopeUI = [];
    sett.ScanScope.forEach(function(item, index, array) {
        item.Enabled = true;
        item.uiFields = getUIFields(item.ScanSettings)
        scanScopeUI.push(item);
    });

    sett.ExcludedFromScanScope.forEach(function(item, index, array) {
        item.Enabled = false;
        item.AreaDesc ='';
        item.ScanSettings = taskDefs.defaultOdsScanSettings();
        item.uiFields = getUIFields(item.ScanSettings)
        scanScopeUI.push(item);
    });

    let data = {
        Form_OnDemand_ScanScope: {
            ScanScopeWithExcludes: scanScopeUI
        },
        Form_OnDemand_Options:{
            UseHeuristicAnalyzer: sett.UseHeuristicAnalyzer,
            HeuristicAnalyzerLevel: sett.HeuristicAnalyzerLevel,
            UseTrustedZoneSettings: sett.UseTrustedZoneSettings,
            UseLowPriorityProcess: sett.UseLowPriorityProcess,
            ScanCriticalAreasTaskFlag: sett.ScanCriticalAreasTaskFlag,
            UseKsn: sett.UseKsn
        }
    }
    return data;
}

function toFormsScanQuarantine(sett) {
    let scanScopeUI = [{
        AreaDesc: '',
        AreaName: '',
        AreaType:1000,
        Enabled: true
    }];
    let data = {
        Form_OnDemand_ScanScope: {
            ScanScopeWithExcludes: scanScopeUI
        },
        Form_OnDemand_Options:{
            UseHeuristicAnalyzer: sett.UseHeuristicAnalyzer,
            HeuristicAnalyzerLevel: sett.HeuristicAnalyzerLevel,
            UseTrustedZoneSettings: sett.UseTrustedZoneSettings,
            UseLowPriorityProcess: sett.UseLowPriorityProcess,
            ScanCriticalAreasTaskFlag: sett.ScanCriticalAreasTaskFlag,
            UseKsn: sett.UseKsn
        }
    }
    return data;
}

function fromFormsOnDemand(changedData, sett) {
    let scanScope = []
    let scanExcludes = []
    changedData.Form_OnDemand_ScanScope.ScanScopeWithExcludes.forEach(function(item, index, array) {
        item.__VersionInfo = [1, 0];
        if(true===item.Enabled ){
            item.ScanSettings = UIFieldsToSettings(item.ScanSettings, item.uiFields);
            delete item.Enabled;
            delete item.uiFields;
            scanScope.push(item);
        }
        else{
            delete item.Enabled;
            delete item.uiFields;
            delete item.ScanSettings;
            delete item.AreaDesc;
            scanExcludes.push(item);
        }
    });

    sett.ScanScope = scanScope;
    sett.ExcludedFromScanScope = scanExcludes;

    sett.UseHeuristicAnalyzer = changedData.Form_OnDemand_Options.UseHeuristicAnalyzer
    sett.HeuristicAnalyzerLevel = changedData.Form_OnDemand_Options.HeuristicAnalyzerLevel
    sett.UseTrustedZoneSettings = changedData.Form_OnDemand_Options.UseTrustedZoneSettings
    sett.UseLowPriorityProcess = changedData.Form_OnDemand_Options.UseLowPriorityProcess
    sett.ScanCriticalAreasTaskFlag = changedData.Form_OnDemand_Options.ScanCriticalAreasTaskFlag
    sett.UseKsn = changedData.Form_OnDemand_Options.UseKsn
    return sett;
}

function fromFormsScanQuarantine(changedData, sett) {
    sett.UseHeuristicAnalyzer = changedData.Form_OnDemand_Options.UseHeuristicAnalyzer
    sett.HeuristicAnalyzerLevel = changedData.Form_OnDemand_Options.HeuristicAnalyzerLevel
    sett.UseTrustedZoneSettings = changedData.Form_OnDemand_Options.UseTrustedZoneSettings
    sett.UseLowPriorityProcess = changedData.Form_OnDemand_Options.UseLowPriorityProcess
    sett.ScanCriticalAreasTaskFlag = changedData.Form_OnDemand_Options.ScanCriticalAreasTaskFlag
    sett.UseKsn = changedData.Form_OnDemand_Options.UseKsn
    return sett;
}

module.exports = {
    toFormsOnDemand,
    toFormsScanQuarantine,
    fromFormsOnDemand,
    fromFormsScanQuarantine
}